#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include "defs.h"
#include "externs.h"
#include "proto.h"
#include "special-functions.h"
#include "mconf.h"  

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
                     R A N D O M  E X C U R S I O N S  T E S T
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

void RandomExcursions(int n)
{
	char   assignment[7];
	int    b, i, j, k, J, state, x;
	int    cycleStart, cycleStop, *cycle, *S_k;
	int    stateX[8] = {-4,-3,-2,-1,1,2,3,4};
	int    counter[8] = {0, 0, 0, 0, 0, 0, 0, 0};
	double p_value, sum, constraint, nu[6][8];
	double pi[10][6] = {
		{0.00000000, 0.00000000, 0.00000000, 0.00000000, 0.00000000, 0.00000000}, 
		{0.5000000000, 0.2500000000, 0.1250000000, 0.06250000000, 0.03125000000, 0.0312500000},
		{0.7500000000, 0.06250000000, 0.04687500000, 0.03515625000, 0.02636718750, 0.0791015625},
		{0.8333333333, 0.02777777778, 0.02314814815, 0.01929012346, 0.01607510288, 0.0803755143},
		{0.8750000000, 0.01562500000, 0.01367187500, 0.01196289063, 0.01046752930, 0.0732727051}
	};
	
#ifdef GEN_TIMING_INFO
	clock_t start, finish;
	FILE	*fp;
	fp = fopen("RndExcursion.txt", "a");

	start = clock();
#endif
	
#if SAVE_RANDOM_EXCURSION_PARAMETERS == 1
	if ( ((cycleInfo = fopen("cycleInfo", "a")) == NULL) ) {
		fprintf(stats[12], "\t\tRandom Excursions:  File cycleInfo could not ");
		fprintf(stats[12], "be opened.\n");
		exit(-1);
	}
#endif

	if ( ((S_k = (int *) calloc(n,sizeof(int))) == NULL) ||
		 ((cycle = (int*) calloc(MAX(1000,n/200),sizeof(int))) == NULL) ) {
		printf("Random Excursions Test:  Insufficent Work Space Allocated.\n");
		if ( S_k != NULL )
			free(S_k);
		if ( cycle != NULL )
			free(cycle);
		return;
	}
	else {
		J = 0; 					/* DETERMINE CYCLES */
		S_k[0] = 2*(int)epsilon[0].b - 1;
		for( i=1; i<n; i++ ) {
			S_k[i] = S_k[i-1] + 2*epsilon[i].b - 1;
			if ( S_k[i] == 0 ) {
				J++;
				if ( J > MAX(1000,n/128) ) {
					printf("ERROR IN FUNCTION randomExcursions:  EXCEEDING THE MAX");
					printf(" NUMBER OF CYCLES EXPECTED\n.");
					fflush(stdout);
					free(S_k);
					free(cycle);
					if ( cycleInfo != NULL )
						fclose(cycleInfo);
					return;
				}
				cycle[J] = i;
			}
		}
		if ( S_k[n-1] != 0 )
			J++;
#if SAVE_RANDOM_EXCURSION_PARAMETERS == 1
		fprintf(cycleInfo,"%d\n", J);
#endif
		if ( RANDOM_EXCURSIONS ) {
			fprintf(stats[12], "\t\t\t  RANDOM EXCURSIONS TEST\n");
			fprintf(stats[12], "\t\t--------------------------------------------\n");
			fprintf(stats[12], "\t\tCOMPUTATIONAL INFORMATION:\n");
			fprintf(stats[12], "\t\t--------------------------------------------\n");
			fprintf(stats[12], "\t\t(a) Number Of Cycles (J) = %04d\n", J);
			fprintf(stats[12], "\t\t(b) Sequence Length (n)  = %d\n", n);
		}
		constraint = MAX(0.005*pow(n,0.5),500);
		if ( J < constraint ) {
			fprintf(stats[12], "\t\t---------------------------------------------");
			fprintf(stats[12], "\n\t\tWARNING:  TEST NOT APPLICABLE.  THERE ARE ");
			fprintf(stats[12], "AN\n\t\t\t  INSUFFICIENT NUMBER OF CYCLES.\n");
			fprintf(stats[12], "\t\t---------------------------------------------");
			fprintf(stats[12], "\n");
			for( i=0; i<8; i++ ) {
				fprintf(results[12], "%f\n", 0.0);
				fprintf(grid, "%d", 0);
				fflush(grid);
			}
		}
		else {
			if ( RANDOM_EXCURSIONS ) {
				fprintf(stats[12], "\t\t(c) Rejection Constraint = %f\n\t", constraint);
				fprintf(stats[12], "\t-------------------------------------------\n");
			}
			cycleStart = 0;
			cycleStop  = cycle[1];
			for( k=0; k<6; k++ )
				for( i=0; i<8; i++ )
					nu[k][i] = 0.0;
			for( j=1; j<=J; j++ ) {                           /* FOR EACH CYCLE */
				for( i=0; i<8; i++ )
					counter[i] = 0;
				for( i=cycleStart; i<cycleStop; i++ ) {
					if ( ((S_k[i] >= 1) && (S_k[i] <= 4)) ||
						 ((S_k[i] >= -4) && (S_k[i] <= -1)) ) {
						if ( S_k[i] < 0 )
							b = 4;
						else
							b = 3;
						counter[S_k[i]+b]++;
					}
				}
				cycleStart = cycle[j]+1;
				if ( j < J ) 
					cycleStop = cycle[j+1];
				else 
					cycleStop = n;

				for( i=0; i<8; i++ ) {
					if ( counter[i] >= 0 && counter[i] <= 4 )
						nu[counter[i]][i]++;
					else if (counter[i] >= 5)
						nu[5][i]++;
				}
			}
			for( i=0; i<8; i++ ) {
				x = stateX[i];
				sum = 0.;
				for( k=0; k<6; k++ )
					sum += pow(nu[k][i] - J*pi[(int)fabs(x)][k],2) / (J*pi[(int)fabs(x)][k]);
				p_value = igamc(2.5, sum/2.);

				if ( isNegative(p_value) || isGreaterThanOne(p_value) )
					fprintf(stats[12], "WARNING:  P_VALUE IS OUT OF RANGE.\n");

				if ( p_value < ALPHA ) {
					state = 0;
					strcpy(assignment, "FAILURE");
				}
				else {
					state = 1;
					strcpy(assignment, "SUCCESS");
				}
				fprintf(stats[12], "%s\t\tx = %2d chi^2 = %9.6f p_value = %f\n",
				assignment, x, sum, p_value);
				fprintf(results[12], "%f\n", p_value);
				fprintf(grid, "%d", state);
				fflush(stats[12]);
				fflush(results[12]);
				fflush(grid);
			}
		}
		fprintf(stats[12], "\n");
		free(S_k);
		free(cycle);
	}

#ifdef GEN_TIMING_INFO
	finish = clock();
	fprintf(fp, "%d\n", finish - start);
	fclose(fp);
#endif

	return;
}
